@@ -1,6 +1,6 @@
 package AI::TensorFlow::Libtensorflow;
 # ABSTRACT: Bindings for Libtensorflow deep learning library
-$AI::TensorFlow::Libtensorflow::VERSION = '0.0.2';
+$AI::TensorFlow::Libtensorflow::VERSION = '0.0.3';
 use strict;
 use warnings;
 
@@ -8,21 +8,26 @@ use AI::TensorFlow::Libtensorflow::Lib;
 
 use AI::TensorFlow::Libtensorflow::DataType;
 use AI::TensorFlow::Libtensorflow::Status;
-
+use AI::TensorFlow::Libtensorflow::TString;
 
 use AI::TensorFlow::Libtensorflow::Buffer;
 use AI::TensorFlow::Libtensorflow::Tensor;
 
 use AI::TensorFlow::Libtensorflow::Operation;
 use AI::TensorFlow::Libtensorflow::Output;
+use AI::TensorFlow::Libtensorflow::Input;
+
+use AI::TensorFlow::Libtensorflow::ApiDefMap;
 
 use AI::TensorFlow::Libtensorflow::ImportGraphDefOptions;
+use AI::TensorFlow::Libtensorflow::ImportGraphDefResults;
 use AI::TensorFlow::Libtensorflow::Graph;
 
 use AI::TensorFlow::Libtensorflow::OperationDescription;
 
 use AI::TensorFlow::Libtensorflow::SessionOptions;
 use AI::TensorFlow::Libtensorflow::Session;
+use AI::TensorFlow::Libtensorflow::DeviceList;
 
 use FFI::C;
 
@@ -38,6 +43,8 @@ sub new {
 
 $ffi->attach( 'Version' => [], 'string' );#}}}
 
+$ffi->attach( 'GetAllOpList' => [], 'TF_Buffer' );
+
 1;
 
 __END__
@@ -50,6 +57,10 @@ __END__
 
 AI::TensorFlow::Libtensorflow - Bindings for Libtensorflow deep learning library
 
+=head1 SYNOPSIS
+
+  use aliased 'AI::TensorFlow::Libtensorflow' => 'Libtensorflow';
+
 =head1 DESCRIPTION
 
 The C<libtensorflow> library provides low-level C bindings
@@ -59,7 +70,8 @@ for TensorFlow with a stable ABI.
 
 =head2 Version
 
-  my $version = $class->Version();
+  my $version = Libtensorflow->Version();
+  like $version, qr/(\d|\.)+/, 'Got version';
 
 B<Returns>
 
@@ -73,6 +85,31 @@ Version number for the C<libtensorflow> library.
 
 B<C API>: L<< C<TF_Version>|AI::TensorFlow::Libtensorflow::Manual::CAPI/TF_Version >>
 
+=head2 GetAllOpList
+
+=over 2
+
+C<<<
+GetAllOpList()
+>>>
+
+=back
+
+  my $buf = Libtensorflow->GetAllOpList();
+  cmp_ok $buf->length, '>', 0, 'Got OpList buffer';
+
+B<Returns>
+
+=over 4
+
+=item L<TFBuffer|AI::TensorFlow::Libtensorflow::Lib::Types/TFBuffer>
+
+Contains a serialized C<OpList> proto for ops registered in this address space.
+
+=back
+
+B<C API>: L<< C<TF_GetAllOpList>|AI::TensorFlow::Libtensorflow::Manual::CAPI/TF_GetAllOpList >>
+
 =head1 AUTHOR
 
 Zakariyya Mughal <zmughal@cpan.org>
