@@ -1,8 +1,9 @@
 package AI::TensorFlow::Libtensorflow::DataType;
 # ABSTRACT: Datatype enum
-$AI::TensorFlow::Libtensorflow::DataType::VERSION = '0.0.2';
+$AI::TensorFlow::Libtensorflow::DataType::VERSION = '0.0.3';
+use strict;
+use warnings;
 use AI::TensorFlow::Libtensorflow::Lib;
-use Scalar::Util qw(dualvar);
 use Const::Exporter;
 
 use Devel::StrictMode;
@@ -50,7 +51,7 @@ Const::Exporter->import(
 		do {
 			%_DTYPES = map {
 				$_ => bless \do {
-					my $value = dualvar( $_ENUM_DTYPE{$_}, $_ )
+					my $value = $_ENUM_DTYPE{$_};
 				}, __PACKAGE__;
 			} keys %_ENUM_DTYPE;
 		},
