@@ -1,12 +1,48 @@
 package AI::TensorFlow::Libtensorflow::Operation;
 # ABSTRACT: An operation
-$AI::TensorFlow::Libtensorflow::Operation::VERSION = '0.0.2';
+$AI::TensorFlow::Libtensorflow::Operation::VERSION = '0.0.3';
+use strict;
+use warnings;
 use namespace::autoclean;
 use AI::TensorFlow::Libtensorflow::Lib qw(arg);
+use AI::TensorFlow::Libtensorflow::Output;
+use AI::TensorFlow::Libtensorflow::Input;
 
 my $ffi = AI::TensorFlow::Libtensorflow::Lib->ffi;
 $ffi->mangler(AI::TensorFlow::Libtensorflow::Lib->mangler_default);
 
+use FFI::C::ArrayDef;
+my $adef = FFI::C::ArrayDef->new(
+	$ffi,
+	name => 'TF_Operation_array',
+	members => [
+		FFI::C::StructDef->new(
+			$ffi,
+			members => [
+				p => 'opaque'
+			]
+		)
+	],
+);
+sub _adef { $adef; }
+sub _as_array {
+	my $class = shift;
+	my $array = $class->_adef->create(0 + @_);
+	for my $idx (0..@_-1) {
+		next unless defined $_[$idx];
+		$array->[$idx]->p($ffi->cast('TF_Operation', 'opaque', $_[$idx]));
+	}
+	$array;
+}
+sub _from_array {
+	my ($class, $array) = @_;
+	[
+		map {
+			$ffi->cast('opaque', 'TF_Operation', $array->[$_]->p);
+		} 0..$array->count-1
+	]
+}
+
 $ffi->attach( [ 'OperationName' => 'Name' ], [
 	arg 'TF_Operation' => 'oper',
 ] => 'string');
@@ -24,23 +60,32 @@ $ffi->attach( [ 'OperationNumOutputs' => 'NumOutputs' ], [
 ] => 'int');
 
 $ffi->attach( [ 'OperationOutputType' => 'OutputType' ] => [
-	# TODO (simplify API)
-	arg 'opaque' => 'TF_Output oper',
-] => 'TF_DataType' );
+	arg 'TF_Output' => 'oper_out',
+] => 'TF_DataType' => sub {
+	my ($xs, $self, $output) = @_;
+	# TODO coerce from LibtfPartialOutput here
+	$xs->($output);
+} );
 
 $ffi->attach( [ 'OperationNumInputs' => 'NumInputs' ] => [
 	arg 'TF_Operation' => 'oper',
 ] => 'int' );
 
 $ffi->attach( [ 'OperationInputType'  => 'InputType' ] => [
-	# TODO (simplify API)
-	arg 'opaque' => 'TF_Input oper_in',
-] => 'TF_DataType');
+	arg 'TF_Input' => 'oper_in',
+] => 'TF_DataType' => sub {
+	my ($xs, $self, $input) = @_;
+	# TODO coerce from LibtfPartialInput here
+	$xs->($input);
+});
 
 $ffi->attach( [ 'OperationNumControlInputs' => 'NumControlInputs' ] => [
 	arg 'TF_Operation' => 'oper',
 ] => 'int' );
 
+$ffi->attach( [ 'OperationNumControlOutputs' => 'NumControlOutputs' ] => [
+	arg 'TF_Operation' => 'oper',
+] => 'int' );
 
 $ffi->attach( [ OperationOutputListLength => 'OutputListLength' ] => [
 	arg 'TF_Operation' => 'oper',
@@ -55,54 +100,72 @@ $ffi->attach( [ 'OperationInputListLength' => 'InputListLength' ] => [
 ] => 'int' );
 
 $ffi->attach( [ 'OperationInput' => 'Input' ] => [
-	arg 'opaque' => 'TF_Input oper_in',
-] => ( 'opaque' , 'TF_Output' )[0] );
+	arg 'TF_Input' => 'oper_in',
+] => 'TF_Output' => sub {
+	my ($xs, $self, $input) = @_;
+	# TODO coerce from LibtfPartialInput here
+	$xs->($input);
+});
 
 $ffi->attach( [ 'OperationAllInputs' => 'AllInputs' ] => [
 	arg 'TF_Operation' => 'oper',
 	# TODO make OutputArray
-	arg 'opaque' => 'TF_Output* inputs',
+	arg 'TF_Output_struct_array' => 'inputs',
 	arg 'int' => 'max_inputs',
-] => 'void' );
+] => 'void' => sub {
+	my ($xs, $oper) = @_;
+	my $max_inputs = $oper->NumInputs;
+	my $inputs = AI::TensorFlow::Libtensorflow::Output->_adef->create(0 + $max_inputs);
+	$xs->($oper, $inputs, $max_inputs);
+	return AI::TensorFlow::Libtensorflow::Output->_from_array($inputs);
+});
+
+$ffi->attach( [ 'OperationGetControlInputs' => 'GetControlInputs' ] => [
+	arg 'TF_Operation' => 'oper',
+	arg 'TF_Operation_array' => 'control_inputs',
+	arg 'int' => 'max_control_inputs',
+] => 'void' => sub {
+	my ($xs, $oper) = @_;
+	my $max_inputs = $oper->NumControlInputs;
+	return [] if $max_inputs == 0;
+	my $inputs = AI::TensorFlow::Libtensorflow::Operation->_adef->create(0 + $max_inputs);
+	$xs->($oper, $inputs, $max_inputs);
+	return AI::TensorFlow::Libtensorflow::Operation->_from_array($inputs);
+});
+
+$ffi->attach( [ 'OperationGetControlOutputs' => 'GetControlOutputs' ] => [
+	arg 'TF_Operation' => 'oper',
+	arg 'TF_Operation_array' => 'control_outputs',
+	arg 'int' => 'max_control_outputs',
+] => 'void' => sub {
+	my ($xs, $oper) = @_;
+	my $max_outputs = $oper->NumControlOutputs;
+	return [] if $max_outputs == 0;
+	my $outputs = AI::TensorFlow::Libtensorflow::Operation->_adef->create(0 + $max_outputs);
+	$xs->($oper, $outputs, $max_outputs);
+	return AI::TensorFlow::Libtensorflow::Operation->_from_array($outputs);
+});
 
 $ffi->attach( [ 'OperationOutputNumConsumers' => 'OutputNumConsumers' ] => [
-	# TODO
-	arg 'opaque' => 'TF_Output oper_out',
-], 'int');
+	arg 'TF_Output' => 'oper_out',
+], 'int' => sub {
+	my ($xs, $self, $output) = @_;
+	# TODO coerce from LibtfPartialOutput here
+	$xs->($output);
+});
 
 $ffi->attach( [ 'OperationOutputConsumers'  => 'OutputConsumers' ] => [
 	# TODO simplify API
-	arg 'opaque' => 'TF_Output oper_out',
-	arg 'opaque' => 'TF_Input* consumers',
-	arg 'int'    => 'max_consumers',
-] => 'int');
-
-
-use FFI::C::ArrayDef;
-my $adef = FFI::C::ArrayDef->new(
-	$ffi,
-	name => 'TF_Operation_array',
-	members => [
-		FFI::C::StructDef->new(
-			$ffi,
-			members => [
-				p => 'opaque'
-			]
-		)
-	],
-);
-sub _adef {
-	$adef;
-}
-sub _as_array {
-	my $class = shift;
-	my $array = $class->_adef->create(0 + @_);
-	for my $idx (0..@_-1) {
-		next unless defined $_[$idx];
-		$array->[$idx]->p($ffi->cast('TF_Operation', 'opaque', $_[$idx]));
-	}
-	$array;
-}
+	arg 'TF_Output' => 'oper_out',
+	arg 'TF_Input_struct_array' => 'consumers',
+	arg 'int'                   => 'max_consumers',
+] => 'int' => sub {
+	my ($xs, $self, $output) = @_;
+	my $max_consumers = $self->OutputNumConsumers( $output );
+	my $consumers = AI::TensorFlow::Libtensorflow::Input->_adef->create( $max_consumers );
+	my $count = $xs->($output, $consumers, $max_consumers);
+	return AI::TensorFlow::Libtensorflow::Input->_from_array( $consumers );
+});
 
 1;
 
@@ -150,6 +213,10 @@ B<C API>: L<< C<TF_OperationInputType>|AI::TensorFlow::Libtensorflow::Manual::CA
 
 B<C API>: L<< C<TF_OperationNumControlInputs>|AI::TensorFlow::Libtensorflow::Manual::CAPI/TF_OperationNumControlInputs >>
 
+=head2 NumControlOutputs
+
+B<C API>: L<< C<TF_OperationNumControlOutputs>|AI::TensorFlow::Libtensorflow::Manual::CAPI/TF_OperationNumControlOutputs >>
+
 =head1 METHODS
 
 =head2 OutputListLength
@@ -168,6 +235,14 @@ B<C API>: L<< C<TF_OperationInput>|AI::TensorFlow::Libtensorflow::Manual::CAPI/T
 
 B<C API>: L<< C<TF_OperationAllInputs>|AI::TensorFlow::Libtensorflow::Manual::CAPI/TF_OperationAllInputs >>
 
+=head2 GetControlInputs
+
+B<C API>: L<< C<TF_OperationGetControlInputs>|AI::TensorFlow::Libtensorflow::Manual::CAPI/TF_OperationGetControlInputs >>
+
+=head2 GetControlOutputs
+
+B<C API>: L<< C<TF_OperationGetControlOutputs>|AI::TensorFlow::Libtensorflow::Manual::CAPI/TF_OperationGetControlOutputs >>
+
 =head2 OutputNumConsumers
 
 B<C API>: L<< C<TF_OperationOutputNumConsumers>|AI::TensorFlow::Libtensorflow::Manual::CAPI/TF_OperationOutputNumConsumers >>
